/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef AIDGE_CPU_OPERATOR_CONCATIMPL_H_
#define AIDGE_CPU_OPERATOR_CONCATIMPL_H_

#include "aidge/backend/OperatorImpl.hpp"
#include "aidge/operator/Concat.hpp"
#include "aidge/utils/Registrar.hpp"
#include "aidge/utils/Types.h"
#include "aidge/backend/cpu/data/GetCPUPtr.h"
#include <memory>
#include <vector>

namespace Aidge {
// class Concat_Op<2>;

// compute kernel registry for forward and backward
class ConcatImplForward_cpu
    : public Registrable<ConcatImplForward_cpu, std::tuple<DataType, DataType>, void(const Concat_Op::Attrs&,
                                                                                     const std::vector<DimSize_t>,
                                                                                     const std::vector<DimSize_t>&,
                                                                                     const std::vector<const void*>,
                                                                                     void*)> {};

class ConcatImplBackward_cpu
    : public Registrable<ConcatImplBackward_cpu, std::tuple<DataType, DataType>, void(const Concat_Op::Attrs&,
                                                                                     const std::vector<DimSize_t>,
                                                                                     const std::vector<DimSize_t>&,
                                                                                     const std::vector<const void*>,
                                                                                     void*)> {};


class ConcatImpl_cpu : public OperatorImpl {
public:
    ConcatImpl_cpu(const Concat_Op& op) : OperatorImpl(op) {}

    static std::unique_ptr<ConcatImpl_cpu> create(const Concat_Op& op) {
        return std::make_unique<ConcatImpl_cpu>(op);
    }

public:
    NbElts_t getNbRequiredData(const IOIndex_t inputIdx) const override final;

    NbElts_t getNbRequiredProtected(const IOIndex_t /*inputIdx*/) const override final;

    NbElts_t getRequiredMemory(const IOIndex_t outputIdx, const std::vector<DimSize_t>& /*inputsSize*/) const override final;

    NbElts_t getNbConsumedData(const IOIndex_t inputIdx) const override final;

    NbElts_t getNbProducedData(const IOIndex_t outputIdx) const override final;

    void updateConsummerProducer() override final;

    void forward() override;

    void backward() override;
};

namespace {
static Registrar<Concat_Op> registrarConcatImpl_cpu("cpu", Aidge::ConcatImpl_cpu::create);
}  // namespace
}  // namespace Aidge

#endif /* AIDGE_CPU_OPERATOR_CONCATIMPL_H_ */
